@@ -5,12 +5,21 @@
 # @Date   : 2022/06/28 21:40
 
 import datetime
+import logging
 import re
+import time
 import warnings
+import os
+from threading import Thread
+import copy
+from functools import reduce
 
 import bs4
 import pytz
 from xml.etree import ElementTree
+from humanfriendly import format_size
+from anytree.importer import DictImporter
+from anytree.exporter import DictExporter
 
 # 过滤这类警告
 warnings.filterwarnings("ignore", category=bs4.MarkupResemblesLocatorWarning, module='bs4')
@@ -22,6 +31,10 @@ if win32com.client.gencache.is_readonly:
     win32com.client.gencache.Rebuild()
 
 from pyxllib.prog.newbie import SingletonForEveryClass
+from pyxllib.prog.pupil import Timeout
+from pyxllib.prog.specialist import tqdm
+from pyxllib.file.specialist import XlPath, get_etag
+from pyxllib.algo.treelib import Node
 
 """
 参考了onepy的实现，做了重构。OnePy：Provides pythonic wrappers around OneNote COM interfaces
@@ -29,32 +42,63 @@ from pyxllib.prog.newbie import SingletonForEveryClass
 
 namespace = "{http://schemas.microsoft.com/office/onenote/2013/onenote}"
 
+# 还未绑定父结点的游离page node，用于进度条子线程
+_free_page_nodes = []
+
+# 缓存文件地址
+CACHE_DIR = XlPath.tempdir() / 'OneNote/SearchCache'
+os.makedirs(CACHE_DIR, exist_ok=True)
+
+# 页面解析结果的缓存，用于解析加速
+_page_parsed_cache = {}
+_page_parsed_cache_file = CACHE_DIR / 'page_parsed_cache_file.pkl'
+if _page_parsed_cache_file.is_file():
+    _page_parsed_cache = _page_parsed_cache_file.read_pkl()
+
+# 用来读取、保存序列化的node数据
+importer = DictImporter()
+exporter = DictExporter()
+
 
 class ONProcess(metaclass=SingletonForEveryClass):
     """ onenote 底层win32的接口 """
 
-    def __init__(self, version=16):
+    def __init__(self, timeout=30):
         """ onenote的win32接口方法是驼峰命名，这个ONProcess做了一层功能封装
         而且估计理论上对所有可以获得的接口都做了封装了
-        """
 
-        try:
-            if version == 16:
-                self.process = win32com.client.gencache.EnsureDispatch('OneNote.Application')
-                self.namespace = "{http://schemas.microsoft.com/office/onenote/2013/onenote}"
-            if version == 15:
-                self.process = win32com.client.gencache.EnsureDispatch('OneNote.Application.15')
-                self.namespace = "{http://schemas.microsoft.com/office/onenote/2013/onenote}"
-            if version == 14:
-                self.process = win32com.client.gencache.EnsureDispatch('OneNote.Application.14')
-                self.namespace = "{http://schemas.microsoft.com/office/onenote/2010/onenote}"
-        except Exception as e:
-            print(e)
-            print("error starting onenote {}".format(version))
+        :param timeout: 读取单个页面的时候，限制用时，单位：秒
+            本来只想限制5秒，但发现会有一些页面特别长，需要多一些时间~
+            再后来发现还有更慢的页面，半分钟的都有，就再改成30秒了
+        """
+        # TODO 这里需要针对不同的OneNote版本做自动化兼容，不要让用户填版本
+        #   因为让用户填版本，会存在多个实例化对象，使用getxml会有各种问题
+        # 目前是支持onenote2016的，但不知道其他版本onenote会怎样
+        self.process = win32com.client.gencache.EnsureDispatch('OneNote.Application')
+        self.namespace = "{http://schemas.microsoft.com/office/onenote/2013/onenote}"
+
+        # 官方原版的实现，但我觉得可以去掉版本号
+        # try:
+        #     if version == 16:
+        #         self.process = win32com.client.gencache.EnsureDispatch('OneNote.Application')
+        #         self.namespace = "{http://schemas.microsoft.com/office/onenote/2013/onenote}"
+        #     if version == 15:
+        #         self.process = win32com.client.gencache.EnsureDispatch('OneNote.Application.15')
+        #         self.namespace = "{http://schemas.microsoft.com/office/onenote/2013/onenote}"
+        #     if version == 14:
+        #         self.process = win32com.client.gencache.EnsureDispatch('OneNote.Application.14')
+        #         self.namespace = "{http://schemas.microsoft.com/office/onenote/2010/onenote}"
+        # except Exception as e:
+        #     # pywintypes.com_error: (-2147221005, '无效的类字符串', None, None)
+        #     # pywintypes.com_error: (-2147221005, '无效的类字符串', None, None)
+        #     print(e)
+        #     print("error starting onenote {}".format(version))
 
         global namespace
         namespace = self.namespace
 
+        self.timeout_seconds = timeout
+
     def get_hierarchy(self, start_node_id="", hierarchy_scope=4):
         """
           HierarchyScope
@@ -117,10 +161,8 @@ class ONProcess(metaclass=SingletonForEveryClass):
           2 - Returns page content with selection markup, but no binary data.
           3 - Returns page content with selection markup and all binary data.
         """
-        try:
+        with Timeout(self.timeout_seconds):
             return self.process.GetPageContent(page_id, "", page_info)
-        except Exception as e:
-            print("Could not get Page Content")
 
     def update_page_content(self, page_changes_xml_in, excpect_last_modified=0):
         try:
@@ -195,24 +237,212 @@ class ONProcess(metaclass=SingletonForEveryClass):
             print("Could not retreive special location")
 
 
-class OneNote(ONProcess, metaclass=SingletonForEveryClass):
-    """ OneNote软件，这是一个单例类 """
+class _CommonMethods:
+    """ 笔记本、分区组、分区、页面 共有的一些成员方法 """
 
-    def __init__(self, version=2016):
+    def init_node(self):
+        return Node(self.name, _category=type(self).__name__)
+
+    @property
+    def ancestors(self):
+        """ 获得所有父结点 """
+        parents = []
+        p = self
+        while getattr(p, 'parent', False):
+            parents.append(p.parent)
+            p = p.parent
+        return reversed(parents)
+
+    @property
+    def abspath_name(self):
+        names = [x.name for x in self.ancestors]
+        names.append(self.name)
+        return '/'.join(names)
+
+    def get_page_num(self):
+        return sum([x.get_page_num() for x in self._children])
+
+    def get_search_tree(self, *, print_mode=True, use_node_cache=True, reparse=False):
+        """ 获得检索树的根节点
+
+        :param print_mode: 输出所包含页面解析进度
+        :param use_node_cache: 检查xml是否有更新
+            默认可以不检查，如果一个widget有_node结点可以直接使用
+        :param reparse: 使用旧的解析过xml的持久化文件数据
+            这个好处是速度快，坏处是如果解析代码功能有更新，这样提取到的是错误的旧的解析数据
+            这个缓存文件的处理规则，为了一些细节优化，也稍复杂~~
         """
-        :param version: onenote版本号，写2010、2013、2016
-            目前不确定支不支持win10自带的onenote、2019、office365
+        global _page_parsed_cache
 
+        # 1 进度条工具，开一个子线程，每秒监控页面解析进度
+        def timer_progress():
+            """ 为了实现这个进度条，还有点技术含量呢，用了子线程和一些trick """
+            global _free_page_nodes
+
+            def dfs(x):
+                cnt = 0
+
+                _category = getattr(x, '_category', '')
+                if _category == 'Page':
+                    cnt += 1
+                elif _category == '':
+                    return cnt
+
+                for y in x.children:
+                    cnt += dfs(y)
+                return cnt
+
+            total = self.get_page_num()
+            _tqdm = tqdm(desc='OneNote解析页面', disable=False, total=total)
+            while not stop_flag:
+                num = dfs(self._node)  # 只读取，不修改self._node，而且进度条稍微有些错没关系，所以不加锁
+                _free_page_nodes = list(filter(lambda x: x.root != self._node, _free_page_nodes))
+                num += len(_free_page_nodes)
+                _tqdm.n = num
+                _tqdm.refresh()
+
+                time.sleep(1)
+
+            # 最后统计一轮
+            num = dfs(self._node)
+            _free_page_nodes = []
+            _tqdm.total = _tqdm.n = num
+            _tqdm.refresh()
+
+            # print(f'一共{total}个页面，实际解析出{num}个页面')
+            # 实际解析成功的页面数
+            return num
+
+        # 2 主线程解析页面的过程，子线程每秒钟展示一次进度情况
+        if reparse and type(self).__name__ == 'OneNote':  # 如果是OneNote层面reparse，直接重置整个缓存文件
+            _page_parsed_cache = {}
+        cache_num = len(_page_parsed_cache)
+
+        if print_mode:
+            stop_flag = False
+            timer_thread = Thread(target=timer_progress)
+            timer_thread.start()
+            root = self._search(use_node_cache=use_node_cache, reparse=reparse)
+            stop_flag = True  # 使用进程里的共享变量，进行主线程和子线程之间的通信
+            timer_thread.join()
+        else:
+            root = self._search(use_node_cache=use_node_cache, reparse=reparse)
+
+        def save_cache():
+            if cache_num != len(_page_parsed_cache):
+                # 如果前后数量不一致，表示有更新内容，重新写入一份缓存文件
+                _page_parsed_cache_file.write_pkl(_page_parsed_cache)
+
+        savefile_thread = Thread(target=save_cache)
+        savefile_thread.start()  # 开子线程去保存文件，和后面的拷贝副本并行处理
+
+        # 3 拷贝副本，并等待保存文件的子线程也执行完
+        root.parent = None
+        savefile_thread.join()
+        return root
+
+    def search(self, pattern, child_depth=0, *,
+               edits=None, reparse=False,
+               return_mode='text', padding_mode=0,
+               print_mode=False, dedent=1):
+        """ 查找内容
+
+        Page、Section、SectionGroup等实际所用的search方法
+
+        :param pattern:
+            text, 检索出现该关键词的node，并展示其相关的上下文内容
+            func, 可以输入自定义函数 check_node(node)->bool，True表示符合检索条件的node
+            re.compile，可以输入编译的正则模式，会使用re.search进行匹配
+        :param int child_depth: 对于检索到的node，向下展开几层子结点
+            -1，表示全部展开
+            0，表示不展开子结点
+            1，只展开直接子结点
+            ...
+        :param return_mode:
+            text，文本展示
+            html，网页富文本
+        """
+        # 1 按照规则检索内容
+        if isinstance(pattern, str):  # 文本关键词检索
+            def check_node(node):
+                # 纯文本部分
+                if pattern in node.name:
+                    return True
+                # 如果纯文本找不到，也会在富文本格式里尝试匹配
+                html_text = getattr(node, '_html_content', '')
+                if html_text and pattern in html_text:
+                    return True
+                return False
+
+        elif isinstance(pattern, re.Pattern):  # 正则检索
+            def check_node(node):
+                if pattern.search(node.name):
+                    return True
+                html_text = getattr(node, '_html_content', '')
+                if html_text and pattern.search(html_text):
+                    return True
+                return False
+        else:  # 自定义检索
+            check_node = pattern
+
+        # 2 更新索引并获得解析树
+        start_time = time.time()
+        edits = edits or []
+        edits = [list(name.split('/')) for name in edits]
+        # 更新易变数据的检索树
+        for path in edits:
+            node = reduce(lambda x, name: x[name], [self] + path)
+            print(node.abspath_name)
+            node.get_search_tree(print_mode=False, use_node_cache=False, reparse=reparse)
+        root = self.get_search_tree(print_mode=print_mode, use_node_cache=True, reparse=reparse)
+        elapsed1 = time.time() - start_time
+
+        # 3 检索内容
+        start_time = time.time()
+        n = root.sign_node(check_node, flag_name='_flag', child_depth=child_depth, reset_flag=True)
+        elapsed2 = time.time() - start_time
+
+        # 4 展示内容
+        texts = [f'更新及获得索引副本：{elapsed1:.2f}秒，内容检索：{elapsed2:.2f}秒，匹配条目数：{n}']
+        if return_mode == 'text':
+            body = root.render(filter_=lambda x: getattr(x, '_flag', 0), dedent=dedent)
+            texts[0] += f'，内容大小：{format_size(len(body.encode()), binary=True)}\n'
+            texts.append(body)
+            return '\n'.join(texts)
+        elif return_mode == 'html':
+            body = root.render_html('_html_content',
+                                    filter_=lambda x: getattr(x, '_flag', 0),
+                                    padding_mode=padding_mode, dedent=dedent)
+            texts[0] += f'，内容大小：{format_size(len(body.encode()), binary=True)}<br/>'
+            texts.append(body)
+            return '<br/>'.join(texts)
+        else:
+            raise ValueError
+
+
+class OneNote(ONProcess, _CommonMethods):
+    """ OneNote软件，这是一个单例类
+
+    注意，从ONProcess继承的OneNote也是单例类
+    但从ONProcess、OneNote生成的是不同的两个对象
+    """
+
+    def __init__(self, timeout=30):
+        """
         如果出现这个错误：This COM object can not automate the makepy process - please run makepy manually for this object
             可以照 https://github.com/varunsrin/one-py 文章末尾的方式操作
             把 HKEY_CLASSES_ROOT\TypeLib\{0EA692EE-BB50-4E3C-AEF0-356D91732725} 的 1.0 删掉
             （这个 KEY ID 值大家电脑上都是一样的）
         """
-        super().__init__({2010: 14, 2013: 15, 2016: 16}.get(version, 16))
+        # trick: 这里有跟单例类有关的一些问题，导致ONProcess需要提前初始化一次
+        super().__init__(timeout)
 
         self.xml = self.get_hierarchy("", 4)
         self.object_tree = ElementTree.fromstring(self.xml)
         self.hierarchy = Hierarchy(self.object_tree)
+        self._children = list(self.hierarchy)
+        self.name = 'onenote'
+        self._node = self.init_node()
 
     def get_page_content(self, page_id):
         page_content_xml = ElementTree.fromstring(super(OneNote, self).get_page_content(page_id))
@@ -246,6 +476,18 @@ class OneNote(ONProcess, metaclass=SingletonForEveryClass):
         """ 通过编号或名称索引获得笔记本 """
         return self.hierarchy[item]
 
+    def _search(self, *, use_node_cache=True, reparse=False):
+        if not self._node.is_leaf and use_node_cache:
+            return self._node
+        else:
+            self._node = self.init_node()
+
+        for x in self._children:
+            cur_node = x._search(use_node_cache=use_node_cache, reparse=reparse)
+            cur_node.parent = self._node
+
+        return self._node
+
 
 class Hierarchy:
 
@@ -289,62 +531,7 @@ class HierarchyNode:
         self.last_modified_time = xml.get("lastModifiedTime")
 
 
-def widget_search(self, pattern, child_depth=1, *, return_mode='text', padding_mode=0):
-    """ 查找内容
-
-    Page、Section、SectionGroup等实际所用的search方法
-
-    :param pattern:
-        text, 检索出现该关键词的node，并展示其相关的上下文内容
-        func, 可以输入自定义函数 check_node(node)->bool，True表示符合检索条件的node
-        re.compile，可以输入编译的正则模式，会使用re.search进行匹配
-    :param int child_depth: 对于检索到的node，向下展开几层子结点
-        -1，表示全部展开
-        0，表示不展开子结点
-        1，只展开直接子结点
-        ...
-    :param return_mode:
-        text，文本展示
-        html，网页富文本
-    """
-    # 1 按照规则检索内容
-    if isinstance(pattern, str):
-        def check_node(node):
-            # 纯文本部分
-            if pattern in node.name:
-                return True
-            # 如果纯文本找不到，也会在富文本格式里尝试匹配
-            html_text = getattr(node, '_html_content', '')
-            if html_text and pattern in html_text:
-                return True
-            return False
-
-    elif isinstance(pattern, re.Pattern):
-        def check_node(node):
-            if pattern.search(node.name):
-                return True
-            html_text = getattr(node, '_html_content', '')
-            if html_text and pattern.search(html_text):
-                return True
-            return False
-    else:
-        check_node = pattern
-
-    root = self._search()
-    root.sign_node(check_node, flag_name='_flag', child_depth=child_depth)
-
-    # 2 展示内容
-    if return_mode == 'text':
-        return root.render(filter_=lambda x: getattr(x, '_flag', 0))
-    elif return_mode == 'html':
-        return root.render_html('_html_content',
-                                filter_=lambda x: getattr(x, '_flag', 0),
-                                padding_mode=padding_mode)
-    else:
-        raise ValueError
-
-
-class Notebook(HierarchyNode):
+class Notebook(HierarchyNode, _CommonMethods):
 
     def __init__(self, xml=None):
         self.xml = xml
@@ -357,6 +544,8 @@ class Notebook(HierarchyNode):
         if xml is not None:
             self.__deserialize_from_xml(xml)
 
+        self._node = self.init_node()
+
     def __deserialize_from_xml(self, xml):
         HierarchyNode.deserialize_from_xml(self, xml)
         self.nickname = xml.get("nickname")
@@ -390,21 +579,20 @@ class Notebook(HierarchyNode):
                     return nb
         return None
 
-    def _search(self):
-        from pyxllib.algo.treelib import Node
+    def _search(self, *, use_node_cache=True, reparse=False):
+        if not self._node.is_leaf and use_node_cache:
+            return self._node
+        else:
+            self._node = self.init_node()
 
-        root = Node('Notebook：' + self.name)
         for x in self._children:
-            cur_node = x._search()
-            cur_node.parent = root
+            cur_node = x._search(use_node_cache=use_node_cache, reparse=reparse)
+            cur_node.parent = self._node
 
-        return root
-
-    def search(self, *args, **kwargs):
-        return widget_search(self, *args, **kwargs)
+        return self._node
 
 
-class SectionGroup(HierarchyNode):
+class SectionGroup(HierarchyNode, _CommonMethods):
     """ 分区组 """
 
     def __init__(self, xml=None, parent_node=None):
@@ -416,6 +604,8 @@ class SectionGroup(HierarchyNode):
         if xml is not None:
             self.__deserialize_from_xml(xml)
 
+        self._node = self.init_node()
+
     def __iter__(self):
         # ckz: 这个遍历的时候，就是OneNote里看到的从左到右的顺序：先所有分区，然后所有分区组
         #
@@ -447,21 +637,26 @@ class SectionGroup(HierarchyNode):
                     return nb
         return None
 
-    def _search(self):
-        from pyxllib.algo.treelib import Node
+    def _search(self, *, use_node_cache=True, reparse=False):
+        if not self._node.is_leaf and use_node_cache:
+            return self._node
+        else:
+            self._node = self.init_node()
 
-        root = Node('SectionGroup：' + self.name)
         for x in self._children:
-            cur_node = x._search()
-            cur_node.parent = root
+            cur_node = x._search(use_node_cache=use_node_cache, reparse=reparse)
+            cur_node.parent = self._node
 
-        return root
+        # 这里多线程效率几乎没差，就不开了
+        # def run_unit(x):
+        #     cur_node = x._search(reset=reset)
+        #     cur_node.parent = self._node
+        # mtqdm(run_unit, self._children, max_workers=2, disable=True)
 
-    def search(self, *args, **kwargs):
-        return widget_search(self, *args, **kwargs)
+        return self._node
 
 
-class Section(HierarchyNode):
+class Section(HierarchyNode, _CommonMethods):
     """ 分区 """
 
     def __init__(self, xml=None, parent_node=None):
@@ -475,6 +670,8 @@ class Section(HierarchyNode):
         if xml is not None:
             self.__deserialize_from_xml(xml)
 
+        self._node = self.init_node()
+
     def __iter__(self):
         for c in self._children:
             yield c
@@ -506,15 +703,22 @@ class Section(HierarchyNode):
                     return nb
         return None
 
-    def _search(self):
-        from pyxllib.algo.treelib import Node
+    def get_page_num(self):
+        return len(self._children)
+
+    def _search(self, *, use_node_cache=True, reparse=False):
+        if not self._node.is_leaf and use_node_cache:
+            return self._node
+        else:
+            self._node = self.init_node()
+
+        page_lv1, page_lv2 = self._node, self._node
 
-        root = Node('Section：' + self.name)
-        page_lv1, page_lv2 = root, root
         for x in self._children:
-            cur_page = x._search()
+            # print(x.name)
+            cur_page = x._search(use_node_cache=use_node_cache, reparse=reparse)
             if x.page_level == '1':
-                cur_page.parent = root
+                cur_page.parent = self._node
                 page_lv1 = cur_page
             elif x.page_level == '2':
                 cur_page.parent = page_lv1
@@ -522,13 +726,10 @@ class Section(HierarchyNode):
             else:
                 cur_page.parent = page_lv2
 
-        return root
-
-    def search(self, *args, **kwargs):
-        return widget_search(self, *args, **kwargs)
+        return self._node
 
 
-class Page:
+class Page(_CommonMethods):
     """ 页面 """
 
     def __init__(self, xml=None, parent_node=None):
@@ -544,6 +745,8 @@ class Page:
         if xml is not None:  # != None is required here, since this can return false
             self.__deserialize_from_xml(xml)
 
+        self._node = self.init_node()  # 供全文检索的树形结点
+
     def __iter__(self):
         for c in self._children:
             yield c
@@ -553,6 +756,13 @@ class Page:
 
         # Get / Set Meta
 
+    @property
+    def root(self):
+        p = self
+        while getattr(p, 'parent', False):
+            p = p.parent
+        return p
+
     def __deserialize_from_xml(self, xml):
         self.xml = xml
         self.name = xml.get("name")
@@ -563,19 +773,43 @@ class Page:
         self.is_currently_viewed = xml.get("isCurrentlyViewed")
         self._children = [Meta(node) for node in xml]
 
-    def getxml(self):
-        """获得页面的"""
-        # 这里 ONProcess() 改成 OneNote 好像有些问题~~
-        return ONProcess().get_page_content(self.id)
+    def get_xml(self, page_info=0):
+        """ 获得页面的xml内容 """
+        # 1 有缓存的文件直接读取
+        prefix = f'{self.id}_{page_info}_'
+        file = CACHE_DIR / (prefix + self.last_modified_time.replace(':', '') + f'.xml')
+
+        if file.is_file():
+            return file.read_text()
+
+        # 2 否则没有缓存，或者文件不是最新，则使用onenote的接口获得文件内容
+        try:
+            res = super(OneNote, onenote).get_page_content(self.id, page_info)
+        except TimeoutError as e:
+            e.args = [e.args[0] + f'\n\t{self.abspath_name} 页面获取失败，请检查可能包含的office公式并删除。' \
+                                  f'并且看下您的OneNote可能无响应了，请重启OneNote。']
+            raise e
+
+        if res is None:
+            logging.warning(f'{self.abspath_name} 未成功提取页面内容')
+        else:
+            # 删除旧时间点的缓存文件，存储新的缓存文件
+            for f in CACHE_DIR.glob(f'{prefix}*.xml'):
+                f.delete()
+            file.write_text(res)
+
+        return res
 
     def browser_xml(self):
         from pyxllib.debug.specialist import browser, XlPath
-        xml = self.getxml()
+        xml = self.get_xml()
         browser(xml, file=XlPath.tempfile('.xml'))
 
-    def parse_oe_tree(self, root=None):
+    def parse_xml(self, root=None, *, page_info=0, reparse=False):
         """ 获得本Page页面的树形结构，返回一个Node根节点
 
+        :param reparse: 默认直接使用缓存里的解析结果，如果设置了reparse则强制重新解析
+
         因为层次结构有多种不同的表现形式
             层次结构A：Outline 文本框
             层次结构B：h1、h2、h3 等标题结构
@@ -584,11 +818,21 @@ class Page:
             比如在遍历tag.contents的时候，因为[层次结构C]的原因，会出现 parent = dfs_parse_node(y, parent) 的较奇怪的写法
             在parse_oe中，parent的层次实现规则，也会较复杂，有些trick
         """
-        from pyxllib.algo.treelib import Node
         from pyxllib.text.xmllib import BeautifulSoup
 
-        xml = self.getxml()
-        soup = BeautifulSoup(xml, 'xml')
+        if root is None:
+            root = Node('root')
+
+        # 1 在一次程序执行中，相同的xml内容解析出的树也是一样的，可以做个缓存
+        xml = self.get_xml(page_info=page_info)
+        etag = get_etag(xml)
+
+        if etag in _page_parsed_cache and not reparse:
+            root.children = importer.import_(_page_parsed_cache[etag]).children
+            return root
+
+        # 2 否则进入正常解析流程
+        soup = BeautifulSoup(xml or '', 'xml')
 
         # self.browser_xml()  # 可以用这个查原始的xml内容
 
@@ -654,7 +898,8 @@ class Page:
                 if x.name == 'QuickStyleDef':
                     style_defs[x['index']] = x['name']
                 elif x.name == 'Title':
-                    parent.root.name = BeautifulSoup(x.T.text, 'lxml').text
+                    # parent.root.name = BeautifulSoup(x.T.text, 'lxml').text
+                    pass
                 elif x.name == 'Outline':
                     # 处理层次结构A
                     nonlocal outline_cnt
@@ -673,23 +918,28 @@ class Page:
                         parent = dfs_parse_node(y, parent)
             return parent  # parent有可能会更新
 
-        if root is None:
-            root = Node('root')
         style_defs = {}
         # trick: outline_cnt不仅用来标记是否有多个Outline需要设中介结点。也记录了当前Outline的编号。
         outline_cnt = 1 if len(soup.find_all('Outline')) > 1 else 0
         dfs_parse_node(soup, root)
+        _page_parsed_cache[etag] = exporter.export(root)
         return root
 
-    def _search(self):
+    def get_page_num(self):
+        return 1
+
+    def _search(self, *, use_node_cache=True, reparse=False):
         """ 先生成所有结点
         """
-        root = self.parse_oe_tree()
-        root.name = self.name
-        return root
+        if not self._node.is_leaf and use_node_cache:
+            # 首先要有孩子结点，不是叶子结点，才表示可能解析过的node，此时开启use_node_cache的话，则不重复解析
+            return self._node
+        else:
+            self._node = self.init_node()
 
-    def search(self, *args, **kwargs):
-        return widget_search(self, *args, **kwargs)
+        self.parse_xml(self._node, reparse=reparse)
+        _free_page_nodes.append(self._node)
+        return self._node
 
 
 class Meta:
@@ -710,7 +960,7 @@ class Meta:
 
     def getxml(self):
         """获得页面的"""
-        return ONProcess().get_page_content(self.id)
+        return super(OneNote, onenote).get_page_content(self.id)
 
 
 class PageContent:
@@ -1022,3 +1272,64 @@ class Image:
 
 
 onenote = OneNote()
+
+
+def start_server(root=None, edits=None, *, port=80):
+    """ 在本地开启一个onenote搜索服务
+
+    :param str root: 要检索的onenote根目录，未设置时默认初始化所有OneNote笔记
+        所有的路径一律用反斜杠/隔开
+        注意这样写是会降低灵活性的
+            一方面本来是可以使用数字直接引用下标的，这种模式下出现的数字会直接判定为是页面名称
+            另一方面路径中本来就可能存在/
+        但这些情况是小概率事件，一般遇到有问题的子目录，改到大目录里定位就好
+            实在不行，读者可以自己复制这个函数自行扩展
+    :param str|list[str] edits: 每次检索前要强制更新检索树的目录
+        str, 用英文逗号隔开多个条目
+        list[str]里的str，统一只要写在root下的相对路径
+
+    >> search_server('核心', ['2022ch4'])
+    >> search_server('共享/陈坤泽', ['杂项', 'CF', '吃土乡/大家的幻想乡'])
+    """
+    from flask import Flask
+    from flask import request
+
+    # 1 初始化检索数据
+    if root:
+        parent = reduce(lambda x, name: x[name], [onenote] + list(root.split('/')))
+    else:
+        parent = onenote
+    parent.get_search_tree(print_mode=True)
+
+    if isinstance(edits, str):
+        # 如果输入是字符串，可能使用命令行启动的，需要转义为list
+        edits = edits.split(',')  # 英文逗号隔开多个参数
+
+    # 2 开服务接口
+    app = Flask(__name__)
+
+    @app.route('/search/onenote', methods=['GET'])
+    def search_onenote():
+        def get_args(key, default=None):
+            return request.args.get(key, default)
+
+        pattern = get_args('pattern')
+        if pattern:
+            # 解析功能细节
+            res = parent.search(pattern, edits=edits,
+                                child_depth=int(get_args('child_depth', 0)),
+                                return_mode=get_args('return_mode', 'html'),
+                                padding_mode=int(get_args('padding_mode', 0)),
+                                dedent=int(get_args('dedent', 1)))
+        else:
+            ref_url = 'http://localhost/search/onenote?pattern=test'
+            return f'请输入检索内容，例如 <a href={ref_url}>{ref_url}</a>'
+        return res
+
+    app.run(host='0.0.0.0', port=port)
+
+
+if __name__ == '__main__':
+    import fire
+
+    fire.Fire()
