@@ -25,7 +25,7 @@ import numpy as np
 import time
 import sys
 
-import tools.infer.utility as utility
+import pyxlpr.ppocr.tools.infer.utility as utility
 from pyxlpr.ppocr.utils.logging import get_logger
 from pyxlpr.ppocr.utils.utility import get_image_file_list, check_and_read_gif
 from pyxlpr.ppocr.data import create_operators, transform
